@@ -130,7 +130,7 @@ static void dump_boot(DOS_FS * fs, struct boot_sector *b, unsigned lss)
     printf("%10u sectors total\n", sectors ? sectors : le32toh(b->total_sect));
 }
 
-static void check_backup_boot(DOS_FS * fs, struct boot_sector *b, int lss)
+static void check_backup_boot(DOS_FS * fs, struct boot_sector *b, unsigned int lss)
 {
     struct boot_sector b2;
 
@@ -145,7 +145,7 @@ static void check_backup_boot(DOS_FS * fs, struct boot_sector *b, int lss)
 	else
 	    printf("  Auto-creating backup boot block.\n");
 	if (!interactive || get_key("12", "?") == '1') {
-	    int bbs;
+	    unsigned int bbs;
 	    /* The usual place for the backup boot sector is sector 6. Choose
 	     * that or the last reserved sector. */
 	    if (le16toh(b->reserved) >= 7 && le16toh(b->info_sector) != 6)
@@ -210,14 +210,15 @@ static void check_backup_boot(DOS_FS * fs, struct boot_sector *b, int lss)
 
 static void init_fsinfo(struct info_sector *i)
 {
+    memset(i, 0, sizeof (struct info_sector));
     i->magic = htole32(0x41615252);
     i->signature = htole32(0x61417272);
     i->free_clusters = htole32(-1);
     i->next_cluster = htole32(2);
-    i->boot_sign = htole16(0xaa55);
+    i->boot_sign = htole32(0xaa550000);
 }
 
-static void read_fsinfo(DOS_FS * fs, struct boot_sector *b, int lss)
+static void read_fsinfo(DOS_FS * fs, struct boot_sector *b, unsigned int lss)
 {
     struct info_sector i;
 
@@ -257,7 +258,7 @@ static void read_fsinfo(DOS_FS * fs, struct boot_sector *b, int lss)
     fs_read(fs->fsinfo_start, sizeof(i), &i);
 
     if (i.magic != htole32(0x41615252) ||
-	i.signature != htole32(0x61417272) || i.boot_sign != htole16(0xaa55)) {
+	i.signature != htole32(0x61417272) || i.boot_sign != htole32(0xaa550000)) {
 	printf("FSINFO sector has bad magic number(s):\n");
 	if (i.magic != htole32(0x41615252))
 	    printf("  Offset %llu: 0x%08x != expected 0x%08x\n",
@@ -267,10 +268,10 @@ static void read_fsinfo(DOS_FS * fs, struct boot_sector *b, int lss)
 	    printf("  Offset %llu: 0x%08x != expected 0x%08x\n",
 		   (unsigned long long)offsetof(struct info_sector, signature),
 		   le32toh(i.signature), 0x61417272);
-	if (i.boot_sign != htole16(0xaa55))
-	    printf("  Offset %llu: 0x%04x != expected 0x%04x\n",
+	if (i.boot_sign != htole32(0xaa550000))
+	    printf("  Offset %llu: 0x%08x != expected 0x%08x\n",
 		   (unsigned long long)offsetof(struct info_sector, boot_sign),
-		   le16toh(i.boot_sign), 0xaa55);
+		   le32toh(i.boot_sign), 0xaa550000);
 	if (interactive)
 	    printf("1) Correct\n2) Don't correct (FSINFO invalid then)\n");
 	else
@@ -328,7 +329,7 @@ void read_boot(DOS_FS * fs)
 {
     struct boot_sector b;
     unsigned total_sectors;
-    unsigned short logical_sector_size, sectors;
+    unsigned int logical_sector_size, sectors;
     off_t fat_length;
     unsigned total_fat_entries;
     off_t data_size;
@@ -415,7 +416,8 @@ void read_boot(DOS_FS * fs)
 	 * much clusers otherwise. */
 	fs->fat_bits = (fs->data_clusters >= FAT12_THRESHOLD) ? 16 : 12;
 	if (fs->data_clusters >= FAT16_THRESHOLD)
-	    die("Too many clusters (%lu) for FAT16 filesystem.", fs->data_clusters);
+	    die("Too many clusters (%lu) for FAT16 filesystem.",
+		    (unsigned long)fs->data_clusters);
 	check_fat_state_bit(fs, &b);
     } else {
 	/* On Atari, things are more difficult: GEMDOS always uses 12bit FATs
@@ -457,7 +459,7 @@ void read_boot(DOS_FS * fs)
 	die("Root directory (%d entries) doesn't span an integral number of "
 	    "sectors.", fs->root_entries);
     if (logical_sector_size & (SECTOR_SIZE - 1))
-	die("Logical sector size (%d bytes) is not a multiple of the physical "
+	die("Logical sector size (%u bytes) is not a multiple of the physical "
 	    "sector size.", logical_sector_size);
 #if 0				/* linux kernel doesn't check that either */
     /* ++roman: On Atari, these two fields are often left uninitialized */
@@ -540,7 +542,7 @@ static void write_volume_label(DOS_FS * fs, char *label)
     offset = find_volume_de(fs, &de);
     if (offset == 0) {
 	created = 1;
-	offset = alloc_rootdir_entry(fs, &de, label);
+	offset = alloc_rootdir_entry(fs, &de, label, 0);
     }
     memcpy(de.name, label, 11);
     de.time = htole16((unsigned short)((mtime->tm_sec >> 1) +
