@@ -3,6 +3,7 @@
    Copyright (C) 1993 Werner Almesberger <werner.almesberger@lrc.di.epfl.ch>
    Copyright (C) 1998 Roman Hodek <Roman.Hodek@informatik.uni-erlangen.de>
    Copyright (C) 2008-2014 Daniel Baumann <mail@daniel-baumann.ch>
+   Copyright (C) 2015 Andreas Bombe <aeb@debian.org>
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -35,11 +36,10 @@
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
+#include <sys/types.h>
 #include <sys/stat.h>
-#include <sys/ioctl.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <linux/fd.h>
 
 #include "fsck.fat.h"
 #include "common.h"
@@ -47,7 +47,7 @@
 
 typedef struct _change {
     void *data;
-    loff_t pos;
+    off_t pos;
     int size;
     struct _change *next;
 } CHANGE;
@@ -55,57 +55,15 @@ typedef struct _change {
 static CHANGE *changes, *last;
 static int fd, did_change = 0;
 
-unsigned device_no;
-
-#ifdef __DJGPP__
-#include "volume.h"		/* DOS lowlevel disk access functions */
-loff_t llseek(int fd, loff_t offset, int whence)
-{
-    if ((whence != SEEK_SET) || (fd == 4711))
-	return -1;		/* only those supported */
-    return VolumeSeek(offset);
-}
-
-#define open OpenVolume
-#define close CloseVolume
-#define read(a,b,c) ReadVolume(b,c)
-#define write(a,b,c) WriteVolume(b,c)
-#else
-loff_t llseek(int fd, loff_t offset, int whence)
-{
-    return (loff_t) lseek64(fd, (off64_t) offset, whence);
-}
-#endif
 
 void fs_open(char *path, int rw)
 {
-    struct stat stbuf;
-
     if ((fd = open(path, rw ? O_RDWR : O_RDONLY)) < 0) {
 	perror("open");
 	exit(6);
     }
     changes = last = NULL;
     did_change = 0;
-
-#ifndef _DJGPP_
-    if (fstat(fd, &stbuf) < 0)
-	pdie("fstat %s", path);
-    device_no = S_ISBLK(stbuf.st_mode) ? (stbuf.st_rdev >> 8) & 0xff : 0;
-#else
-    if (IsWorkingOnImageFile()) {
-	if (fstat(GetVolumeHandle(), &stbuf) < 0)
-	    pdie("fstat image %s", path);
-	device_no = 0;
-    } else {
-	/* return 2 for floppy, 1 for ramdisk, 7 for loopback  */
-	/* used by boot.c in Atari mode: floppy always FAT12,  */
-	/* loopback / ramdisk only FAT12 if usual floppy size, */
-	/* harddisk always FAT16 on Atari... */
-	device_no = (GetVolumeHandle() < 2) ? 2 : 1;
-	/* telling "floppy" for A:/B:, "ramdisk" for the rest */
-    }
-#endif
 }
 
 /**
@@ -117,26 +75,22 @@ void fs_open(char *path, int rw)
  * @param[in]   size    Number of bytes to read
  * @param[out]  data    Where to put the data read
  */
-void fs_read(loff_t pos, int size, void *data)
+void fs_read(off_t pos, int size, void *data)
 {
     CHANGE *walk;
     int got;
 
-    if (llseek(fd, pos, 0) != pos)
-	pdie("Seek to %lld", pos);
+    if (lseek(fd, pos, 0) != pos)
+	pdie("Seek to %lld", (long long)pos);
     if ((got = read(fd, data, size)) < 0)
-	pdie("Read %d bytes at %lld", size, pos);
+	pdie("Read %d bytes at %lld", size, (long long)pos);
     if (got != size)
-	die("Got %d bytes instead of %d at %lld", got, size, pos);
+	die("Got %d bytes instead of %d at %lld", got, size, (long long)pos);
     for (walk = changes; walk; walk = walk->next) {
 	if (walk->pos < pos + size && walk->pos + walk->size > pos) {
 	    if (walk->pos < pos)
-		memcpy(data, (char *)walk->data + pos - walk->pos, min(size,
-								       walk->
-								       size -
-								       pos +
-								       walk->
-								       pos));
+		memcpy(data, (char *)walk->data + pos - walk->pos,
+		       min(size, walk->size - pos + walk->pos));
 	    else
 		memcpy((char *)data + walk->pos - pos, walk->data,
 		       min(walk->size, size + pos - walk->pos));
@@ -144,33 +98,33 @@ void fs_read(loff_t pos, int size, void *data)
     }
 }
 
-int fs_test(loff_t pos, int size)
+int fs_test(off_t pos, int size)
 {
     void *scratch;
     int okay;
 
-    if (llseek(fd, pos, 0) != pos)
-	pdie("Seek to %lld", pos);
+    if (lseek(fd, pos, 0) != pos)
+	pdie("Seek to %lld", (long long)pos);
     scratch = alloc(size);
     okay = read(fd, scratch, size) == size;
     free(scratch);
     return okay;
 }
 
-void fs_write(loff_t pos, int size, void *data)
+void fs_write(off_t pos, int size, void *data)
 {
     CHANGE *new;
     int did;
 
     if (write_immed) {
 	did_change = 1;
-	if (llseek(fd, pos, 0) != pos)
-	    pdie("Seek to %lld", pos);
+	if (lseek(fd, pos, 0) != pos)
+	    pdie("Seek to %lld", (long long)pos);
 	if ((did = write(fd, data, size)) == size)
 	    return;
 	if (did < 0)
-	    pdie("Write %d bytes at %lld", size, pos);
-	die("Wrote %d bytes instead of %d at %lld", did, size, pos);
+	    pdie("Write %d bytes at %lld", size, (long long)pos);
+	die("Wrote %d bytes instead of %d at %lld", did, size, (long long)pos);
     }
     new = alloc(sizeof(CHANGE));
     new->pos = pos;
@@ -191,7 +145,7 @@ static void fs_flush(void)
     while (changes) {
 	this = changes;
 	changes = changes->next;
-	if (llseek(fd, this->pos, 0) != this->pos)
+	if (lseek(fd, this->pos, 0) != this->pos)
 	    fprintf(stderr,
 		    "Seek to %lld failed: %s\n  Did not write %d bytes.\n",
 		    (long long)this->pos, strerror(errno), this->size);
